@@ -1,4 +1,4 @@
-from enum import Enum, IntEnum
+from enum import IntEnum
 from typing import Tuple
 
 import numpy as np
@@ -6,15 +6,6 @@ import scipy.special as scipy_bessel
 from scipy import interpolate
 
 
-class BesselType(Enum):
-    """Enum class specifying the type of Bessel function to use in
-    :func:`.bessel_zeros`"""
-    JN = 1  #: Bessel function of the first kind :math:`J_n`
-    YN = 2  #: Bessel function of the second kind :math:`Y_n`
-    JNP = 3  #: Derivative of Bessel function of the first kind :math:`J'_n`
-    YNP = 4  #: Derivative of Bessel function of the second kind :math:`Y'_n`
-
-
 class HankelTransformMode(IntEnum):
     """Enum class specifying the scaling of the functions used in the Hankel
     transform. See :ref:`Scaling <scaling>` for details of usage"""
@@ -27,12 +18,41 @@ class HankelTransformMode(IntEnum):
 class HankelTransform:
     r"""The main class for performing Hankel Transforms
 
+        For the QDHT to work, the function must be sampled a specific points, which this class generates
+        and stores in :attr:`HankelTransform.r`. Any transform on this grid will be sampled at points
+        :attr:`.HankelTransform.v` (frequency space) or equivalently :attr:`.HankelTransform.kr`
+        (angular frequency or wavenumber space).
+
+        The constructor has one required argument (``order``). The remaining four arguments offer
+        three different ways of specifying the radial (and therefore implicitly the frequency) points:
+
+        1. Supply both a maximum radius ``r_max`` and number of transform points ``n_points``
+        2. Supply the original (often equally spaced) ``radial_grid`` on which you have currently
+           have sample points. This approach allows easy conversion from the original grid using
+           :meth:`.HankelTransform.to_transform_r()`. ``t = HankelTransform(order, radial_grid=r)``
+           is effectively equivalent to ``t = HankelTransform(order, n_points=r.size, r_max=np.max(r))``
+           except for the fact the the original radial grid is stored in the :class:`.HankelTransform`
+           object for use in :meth:`~.HankelTransform.to_transform_r` and
+           :meth:`~.HankelTransform.to_original_r`.
+        3. Supply the original (often equally spaced) :math:`k`-space grid on which you
+           have currently have sample points. This is most use if you intend to do inverse
+           transforms. It allows easy conversion to and from the original grid using
+           :meth:`~.HankelTransform.to_original_k()` and :meth:`~.HankelTransform.to_transform_k()`.
+           As in option 2, :attr:`.HankelTransform.n_points` is determined by ``k_grid.size``.
+           :attr:`HankelTransform.r_max` is determined in a more complex way from ``np.max(k_grid)``.
+
         :parameter order: Transform order :math:`p`
         :type order: :class:`int`
-        :parameter max_radius: Radial extent of transform :math:`r_\textrm{max}`
+        :parameter max_radius: (Optional) Radial extent of transform :math:`r_\textrm{max}`
         :type max_radius: :class:`float`
-        :parameter n_points: Number of sample points :math:`N`
+        :parameter n_points: (Optional) Number of sample points :math:`N`
         :type n_points: :class:`int`
+        :parameter radial_grid: (Optional) The radial grid that will be used to sample input functions
+            it is used to set `N` and :math:`r_\textrm{max}` by ``n_points = radial_grid.size`` and
+            ``r_max = np.max(radial_grid)``
+        :type radial_grid: :class:`numpy.ndarray`
+        :parameter k_grid: (Optional) Number of sample points :math:`N`
+        :type k_grid: :class:`numpy.ndarray`
 
         :ivar alpha: The first :math:`N` Roots of the :math:`p` th order Bessel function.
         :ivar alpha_n1: (N+1)th root :math:`\alpha_{N1}`
@@ -52,21 +72,15 @@ class HankelTransform:
             Manuel Guizar-Sicairos and Julio C. Guitierrez-Vega
             J. Opt. Soc. Am. A **21** (1) 53-58 (2004)
 
-        The algorithm also calls the function:
-
-        .. code-block:: python
-
-            alpha = bessel_zeros(BesselType.JN, order, n_points+1,)
-
-        where ``order`` and ``n_points`` are defined above, to calculate the roots of the bessel
-        function.
+        The algorithm also calls the function :func:`scipy.special.jn_zeros` to calculate
+        the roots of the bessel function.
 
         .. _scaling:
 
         .. admonition:: Scaling
 
             The :meth:`.HankelTransform.qdht` and :meth:`~.HankelTransform.iqdht` functions can accept
-            ``scaling`` argument (an instance of :class:`HankelTransformMode`) which allows
+            a ``scaling`` argument (an instance of :class:`HankelTransformMode`) which allows
             skipping the scaling that is otherwise necessary in the
             algorithm. For a use case when the same function is transformed multiple times,
             this can increase the speed of the algorithm. See
@@ -131,12 +145,12 @@ class HankelTransform:
         self._original_k_grid = k_grid
 
         # Calculate N+1 roots must be calculated before max_radius can be derived from k_grid
-        alpha = bessel_zeros(BesselType.JN, self.order, self.n_points + 1)
+        alpha = scipy_bessel.jn_zeros(self.order, self.n_points + 1)
         self.alpha = alpha[0:-1]
         self.alpha_n1 = alpha[-1]
 
         if k_grid is not None:
-            v_max = np.max(k_grid)
+            v_max = np.max(k_grid) / (2 * np.pi)
             max_radius = self.alpha_n1 / (2 * np.pi * v_max)
         self._max_radius = max_radius
 
@@ -197,7 +211,7 @@ class HankelTransform:
         grid points used to construct the ``HankelTransform`` object onto the grid required
         of use in the QDHT algorithm.
 
-        If the the ``HankelTransform`` object was constructed with a (say) equally spaced
+        If the the ``HankelTransform`` object was constructed with a (say) equally-spaced
         grid in radius, then it needs the function to transform to be sampled at a specific
         grid before it can be passed to :meth:`.HankelTransform.qdht`. This method provides
         a convenient way of doing this.
@@ -217,7 +231,7 @@ class HankelTransform:
         ``self.r`` (as returned by :meth:`HankelTransform.iqdht`) back onto the original grid
         used to construct the ``HankelTransform`` object.
 
-        If the the ``HankelTransform`` object was constructed with a (say) equally spaced
+        If the the ``HankelTransform`` object was constructed with a (say) equally-spaced
         grid in radius, it may be useful to convert back to this grid after a IQDHT.
         This method provides a convenient way of doing this.
 
@@ -235,8 +249,8 @@ class HankelTransform:
         grid points used to construct the ``HankelTransform`` object onto the grid required
         of use in the IQDHT algorithm.
 
-        If the the ``HankelTransform`` object was constructed with a (say) equally spaced
-        grid in k, then it needs the function to transform to be sampled at a specific
+        If the the ``HankelTransform`` object was constructed with a (say) equally-spaced
+        grid in :math:`k`, then it needs the function to transform to be sampled at a specific
         grid before it can be passed to :meth:`.HankelTransform.iqdht`. This method provides
         a convenient way of doing this.
 
@@ -256,8 +270,8 @@ class HankelTransform:
         ``self.k`` (as returned by :meth:`HankelTransform.qdht`) back onto the original grid
         used to construct the ``HankelTransform`` object.
 
-        If the the ``HankelTransform`` object was constructed with a (say) equally spaced
-        grid in k, it may be useful to convert back to this grid after a QDHT.
+        If the the ``HankelTransform`` object was constructed with a (say) equally-spaced
+        grid in :math:`k`, it may be useful to convert back to this grid after a QDHT.
         This method provides a convenient way of doing this.
 
         :parameter function: The function to be interpolated. Specified at the radial points
@@ -287,7 +301,8 @@ class HankelTransform:
 
         :parameter fr: Function in real space as a function of radius (sampled at ``self.r``)
         :type fr: :class:`numpy.ndarray`
-        :parameter scaling: (optional) Parameter to control the scaling of input and output. See Scaling above
+        :parameter scaling: (optional) Parameter to control the scaling of input and output. See
+            :ref:`Scaling <scaling>` above
         :type scaling: :class:`.HankelTransformMode`
 
         :return: Function in frequency space (sampled at ``self.v``)
@@ -321,7 +336,8 @@ class HankelTransform:
 
         :parameter fv: Function in frequency space (sampled at self.v)
         :type fv: :class:`numpy.ndarray`
-        :parameter scaling: (optional) Parameter to control the scaling of input and output. See Scaling above
+        :parameter scaling: (optional) Parameter to control the scaling of input and output. See
+            :ref:`Scaling <scaling>` above
         :type scaling: :class:`.HankelTransformMode`
 
         :return: Radial function (sampled at self.r) = IHT(fv)
@@ -355,46 +371,6 @@ class HankelTransform:
         return jr, jv
 
 
-def bessel_zeros(bessel_function_type: BesselType, bessel_order: int, n_zeros: int) -> np.ndarray:
-    """Find the first :code:`n_zeros` zeros of a Bessel function of order :code:`bessel_order`.
-
-    The type of the Bessel function can be selected using the ``bessel_function_type`` parameter.
-    It can be :math:`J_n`, :math:`Y_n`, :math:`J'_n`, or :math:`Y'_n`.
-
-    This function is a convenience wrapper for :func:`scipy.special.jn_zeros`,
-    :func:`~scipy.special.yn_zeros`, :func:`~scipy.special.jnp_zeros`, and
-    :func:`~scipy.special.ynp_zeros`. It calls those functions to the actual
-    calculation.
-
-    :parameter bessel_function_type: :class:`.BesselType` object specifying the
-        type of Bessel function for which to find the zeros
-    :type bessel_function_type: :class:`.BesselType`
-    :parameter bessel_order: Bessel order The order of the Bessel function :math:`n`
-    :type bessel_order: :class:`int`
-    :parameter n_zeros:	Number of zeros to find
-    :type n_zeros: :class:`int`
-
-    :return: Zeros of the Bessel function
-    :rtype: :class:`numpy.ndarray`
-
-    """
-    if bessel_function_type == BesselType.JN:
-        return scipy_bessel.jn_zeros(bessel_order, n_zeros)
-    elif bessel_function_type == BesselType.YN:
-        return scipy_bessel.yn_zeros(bessel_order, n_zeros)
-    elif bessel_function_type == BesselType.JNP:
-        zeros = scipy_bessel.jnp_zeros(bessel_order, n_zeros)
-        if bessel_order == 0:
-            # to match Matlab implementation
-            zeros[1:] = zeros[:-1]
-            zeros[0] = 0
-        return zeros
-    elif bessel_function_type == BesselType.YNP:
-        return scipy_bessel.ynp_zeros(bessel_order, n_zeros)
-    else:
-        raise NotImplementedError
-
-
 def _spline(x0: np.ndarray, y0: np.ndarray, x: np.ndarray) -> np.ndarray:
     f = interpolate.interp1d(x0, y0, axis=0, fill_value='extrapolate', kind='cubic')
     return f(x)
